import torch
from ailab.atp_finetuner.model.model import AILabModel
from ailab.atp_finetuner.build import ModelRg
from ailab.atp_finetuner.constant import Task, Model

@ModelRg.register((Task.text_to_image, Model.stable_diffusion_xl_base_1))
class SDXLModel(AILabModel):
    def __init__(self, model: any):
        self.noise_scheduler = None
        self.text_encoder_one = None
        self.text_encoder_two = None
        self.vae = None
        self.unet = None
        super().__init__(model)

    def forward(self,**kwargs):
        text_encoder_one = self.text_encoder_one
        text_encoder_two = self.text_encoder_two
        vae = self.vae
        unet = self.unet
        accelerator = self.accelerator.accelerator_ins

        vae.requires_grad_(False)
        text_encoder_one.requires_grad_(False)
        text_encoder_two.requires_grad_(False)
        unet.requires_grad_(False)
        # For mixed precision training we cast the text_encoder and vae weights to half-precision
        # as these models are only used for inference, keeping weights in full precision is not required.
        weight_dtype = torch.float32
        if accelerator.mixed_precision == "fp16":
            weight_dtype = torch.float16
        elif accelerator.mixed_precision == "bf16":
            weight_dtype = torch.bfloat16

        # Move unet, vae and text_encoder to device and cast to weight_dtype
        unet.to(accelerator.device, dtype=weight_dtype)
        vae.to(accelerator.device, dtype=torch.float32)
        text_encoder_one.to(accelerator.device, dtype=weight_dtype)
        text_encoder_two.to(accelerator.device, dtype=weight_dtype)
        unet.enable_xformers_memory_efficient_attention()

        from peft import LoraConfig
        unet_lora_config = LoraConfig(
            r=4,
            lora_alpha=4,
            init_lora_weights="gaussian",
            target_modules=["to_k", "to_q", "to_v", "to_out.0"],
        )
        unet.add_adapter(unet_lora_config)
    
    @classmethod
    def build_model(cls, device_name:str, model_name:str, model_dir:str, **kwargs):
        from transformers import PretrainedConfig
        def import_model_class_from_model_name_or_path(
            pretrained_model_name_or_path: str, revision: str = None, subfolder: str = "text_encoder"
        ):
            text_encoder_config = PretrainedConfig.from_pretrained(
                pretrained_model_name_or_path, subfolder=subfolder, revision=revision
            )
            model_class = text_encoder_config.architectures[0]

            if model_class == "CLIPTextModel":
                from transformers import CLIPTextModel

                return CLIPTextModel
            elif model_class == "CLIPTextModelWithProjection":
                from transformers import CLIPTextModelWithProjection

                return CLIPTextModelWithProjection
            else:
                raise ValueError(f"{model_class} is not supported.")
    
        from diffusers import AutoencoderKL,DDPMScheduler,StableDiffusionXLPipeline,UNet2DConditionModel
        model_name_or_dir = model_name if model_dir is None else model_dir
        text_encoder_cls_one = import_model_class_from_model_name_or_path(model_name_or_dir)
        text_encoder_cls_two = import_model_class_from_model_name_or_path(model_name_or_dir,  subfolder="text_encoder_2")
        noise_scheduler = DDPMScheduler.from_pretrained(model_name_or_dir, subfolder="scheduler")
        text_encoder_one = text_encoder_cls_one.from_pretrained(
            model_name_or_dir, subfolder="text_encoder"
        )
        text_encoder_two = text_encoder_cls_two.from_pretrained(
            model_name_or_dir, subfolder="text_encoder_2"
        )
        vae = AutoencoderKL.from_pretrained(
            model_name_or_dir,
            subfolder="vae",
        )
        unet = UNet2DConditionModel.from_pretrained(
            model_name_or_dir, subfolder="unet",
        )
        sd_cls = cls(unet)
        sd_cls.noise_scheduler = noise_scheduler
        sd_cls.text_encoder_one = text_encoder_one
        sd_cls.text_encoder_two = text_encoder_two
        sd_cls.vae = vae
        sd_cls.unet = unet
        return sd_cls
    
    def get_inside_models(self, model_type:str):
        if model_type == "noise_scheduler":
            return self.noise_scheduler
        elif model_type == "text_encoder_one":
            return self.text_encoder_one
        elif model_type == "text_encoder_two":
            return self.text_encoder_two
        elif model_type == "vae":
            return self.vae
        elif model_type == "unet":
            return self.unet
        else :
            return None
